#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <gl\gl.h>

void Draw (HWND hWnd)
{
	//  ,        
	//    
	RECT rect;
	GetClientRect (hWnd, &rect);
	GLdouble width = rect.right - rect.left;
	GLdouble height = rect.bottom - rect.top;
	GLdouble left, top, right, bottom;
	if (width > height) {
		top = 1.0;
		bottom = 0.0;
		GLdouble offset = (width / height - 1.0) / 2.0;
		left = - offset;
		right = 1.0 + offset;
	} else {
		left = 0.0;
		right = 1.0;
		GLdouble offset = (height / width - 1.0) / 2.0;
		bottom = - offset;
		top = 1.0 + offset;
	}
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	glOrtho (left, right, bottom, top, -1.0, 1.0);

	//   
	glClearColor (1.0f, 1.0f, 1.0f, 1.0f);
	glClear (GL_COLOR_BUFFER_BIT);

	//    
	glColor3f (0.7f, 0.7f, 0.7f);
	glRectf (0.01f, 0.01f, 0.99f, 0.99f);
	for (int i = 1; i <= 8; ++i) {
		for (int j = 1; j <= 8; ++j) {
			//   
			if ((i ^ j) & 1)
				glColor3f (0.9f, 0.9f, 0.85f);
			else
				glColor3f (0.35f, 0.4f, 0.4f);
			//   
			glRectf (i * 0.1f, j * 0.1f, (i + 1) * 0.1f, (j + 1) * 0.1f);
		}
	}
	//     
	wglUseFontBitmaps (wglGetCurrentDC (), 0, 256, 1000);
	glListBase (1000); 
	glColor3f (0.0f, 0.0f, 0.0f);
	char letter;
	for (char c = 0; c < 8; ++c) {
		//  A-H
		letter = c + 'A';
		glRasterPos2f (0.15f + c * 0.1f, 0.05f);
		glCallLists (1, GL_UNSIGNED_BYTE, &letter); 
		glRasterPos2f (0.15f + c * 0.1f, 0.92f);
		glCallLists (1, GL_UNSIGNED_BYTE, &letter); 
		//  1-8
		letter = c + '1';
		glRasterPos2f (0.06f, 0.13f + c * 0.1f);
		glCallLists (1, GL_UNSIGNED_BYTE, &letter); 
		glRasterPos2f (0.92f, 0.13f + c * 0.1f);
		glCallLists (1, GL_UNSIGNED_BYTE, &letter); 
	}

	//   
	glFinish ();
	SwapBuffers (wglGetCurrentDC ());
}

LRESULT CALLBACK WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		case WM_CREATE:
		{
			//  OpenGL
			HDC hDC = GetDC (hWnd);
			PIXELFORMATDESCRIPTOR pfd;
			ZeroMemory (&pfd, sizeof (pfd));
			pfd.nSize = sizeof (PIXELFORMATDESCRIPTOR);
			pfd.nVersion = 1;
			pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
			pfd.iPixelType = PFD_TYPE_RGBA;
			pfd.cColorBits = 32;
			pfd.cDepthBits = 32;
			pfd.iLayerType = PFD_MAIN_PLANE;
			int i = ChoosePixelFormat (hDC, &pfd);
			SetPixelFormat (hDC, i, &pfd);
			DescribePixelFormat (hDC, i, sizeof (pfd), &pfd);
			HGLRC hGlrc = wglCreateContext (hDC);
			if (!hGlrc)
				return -1;
			wglMakeCurrent (hDC, hGlrc);
			return 0;
		}

		case WM_PAINT:
			//  
			if (GetUpdateRect (hWnd, NULL, FALSE)) {
				PAINTSTRUCT ps;
				BeginPaint (hWnd, &ps);
				//  
				Draw (hWnd);
				EndPaint (hWnd, &ps);
			}
			return 0;

		case WM_SIZE:
		{
			//  OpenGL    
			RECT rect;
			GetClientRect (hWnd, &rect);
			glViewport (rect.left, rect.top, rect.right, rect.bottom);
			break;
		}

		case WM_DESTROY:
		{
			//   OpenGL
			HGLRC hGlrc = wglGetCurrentContext ();
			if (hGlrc) {
				HDC hDC = wglGetCurrentDC ();
				wglMakeCurrent (NULL, NULL);
				ReleaseDC (hWnd, hDC);
				wglDeleteContext (hGlrc);
			}
			PostQuitMessage (0);
			return 0;
		}
	}
	return DefWindowProc(hWnd, message, wParam, lParam);
}

int APIENTRY WinMain (HINSTANCE hInstance, HINSTANCE, LPTSTR, int)
{
	//   
	WNDCLASSEX wcex;
	ZeroMemory (&wcex, sizeof (wcex));
	wcex.cbSize = sizeof (WNDCLASSEX); 
	wcex.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wcex.lpfnWndProc = WndProc;
	wcex.hInstance = hInstance;
	wcex.hIcon = LoadIcon (NULL, IDI_ASTERISK);
	wcex.hCursor = LoadCursor (NULL, IDC_CROSS);
	wcex.hbrBackground = NULL;
	wcex.lpszClassName = "lb041219";
	if (RegisterClassEx (&wcex)) {
		//  
		HWND hWnd = CreateWindowEx (WS_EX_APPWINDOW,
			wcex.lpszClassName, wcex.lpszClassName,
			WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE,
			CW_USEDEFAULT, CW_USEDEFAULT, 600, 400,
			GetDesktopWindow (), NULL, hInstance, NULL);
		if (hWnd) {
			//  
			ShowWindow (hWnd, SW_NORMAL);
			UpdateWindow (hWnd);
			//   
			MSG msg;
			while (GetMessage (&msg, NULL, 0, 0)) {
				TranslateMessage (&msg);
				DispatchMessage (&msg);
			}
		}
	}
	return 0;
}
